//      
//  

#if !defined(_FILTERS_H_)
#define _FILTERS_H_

#include <math.h>
#include "ProgressDlg.h"

#define PI	3.14159265358979323846		//  Pi
#define PITCH_SEG	1024
#define NOISE_SEG	512

//       
//  

struct SampleData {
	int N;
	char *data;
	int Is16bit;
	int IsStereo;
	int Channel;
	SampleData(int N, char *data, int Is16bit, int IsStereo, int Channel)
	{
		SampleData::N = N;
		SampleData::data = data;
		SampleData::Is16bit = Is16bit;
		SampleData::IsStereo = IsStereo;
		SampleData::Channel = Channel;
	}
	int GetSampleSize()
	{
		return (1 << Is16bit) << IsStereo;
	}
	double GetSample(int index)
	{
		index <<= IsStereo;
		index += Channel;
		if (Is16bit)
			return ((short *)data)[index];
		else
			return (unsigned char)data[index] - 128;
	}
	void SetSample(int index, double val)
	{
		index <<= IsStereo;
		index += Channel;
		if (Is16bit)
		{
			if (val > 32767.0) val = 32767.0;
			if (val < -32768.0) val = -32768.0;
			((short *)data)[index] = short(val);
		}
		else
		{
			if (val > 127.0) val = 127.0;
			if (val < -128.0) val = -128.0;
			data[index] = (unsigned char)(val + 128);
		}
	}
};

class Complex {
	double re, im;
public:
	Complex()
	{
		re = im = 0.0;
	}
	Complex(double re, double im)
	{
		Complex::re = re;
		Complex::im = im;
	}
	Complex(double re)
	{
		Complex::re = re;
		Complex::im = 0.0;
	}
	Complex operator =(const Complex c)
	{
		re = c.re;
		im = c.im;
	}
	friend Complex operator +(const Complex c1, const Complex c2)
	{
		return Complex(c1.re + c2.re, c1.im + c2.im);
	}
	friend Complex operator -(const Complex c1, const Complex c2)
	{
		return Complex(c1.re - c2.re, c1.im - c2.im);
	}
	friend Complex operator *(const Complex c1, const Complex c2)
	{
		return Complex(c1.re * c2.re - c1.im * c2.im, c1.re * c2.im + c2.re * c1.im);
	}
	friend Complex operator /(const Complex c1, const Complex c2)
	{
		return Complex((c1.re * c2.re + c1.im * c2.im) / (c2.re * c2.re + c2.im * c2.im),
					   (c2.re * c1.im - c1.re * c2.im) / (c2.re * c2.re + c2.im * c2.im));
	}
	Complex operator +=(const Complex c)
	{
		re += c.re;
		im += c.im;
		return *this;
	}
	Complex operator -=(const Complex c)
	{
		re -= c.re;
		im -= c.im;
		return *this;
	}
	Complex operator *=(const Complex c)
	{
		double t = re * c.re - im * c.im;
		im = re * c.im + c.re * im;
		re = t;
		return *this;
	}
	Complex operator /=(const Complex c)
	{
		double t = (re * c.re + im * c.im) / (c.re * c.re + c.im * c.im);
		im = (c.re * im - re * c.im) / (c.re * c.re + c.im * c.im);
		re = t;
		return *this;
	}
	double Re() { return re; }
	double Im() { return im; }
	double Mod() { return sqrt(re * re + im * im); }
	double Arg()
	{
		double C;
		if (re > 0)
			C = 0.0;
		else
		{
			if (im > 0) C = -M_PI;
			else C = M_PI;
		}
		if (!re)
			return M_PI;
		double t = atan(im / re) + C;
//		if (fabs(t) > 2 * M_PI) t -= floor(t / (2 * M_PI)) * 2 * M_PI;
		return t;
	}
	void FromPolar(double r, double w)
	{
		re = r * cos(w);
		im = r * sin(w);
	}
};

//    

struct UserFilter {
	int Bands[10];
	int KernelSize;
	int WindowType;
};

//  

//-----------------------------------------------------------

//    
void ForwardFFT(int N, double *re, double *im);

//    
void InverseFFT(int N, double *re, double *im);

//    
void FFT(int N, double *re, double *im);

//     
void DFT(int N, double *re, double *im);

//    (  )
void Convolution(int n1, double *x, int n2, double *h, double *y);

//    (  )
void FFTConvolution(int n1, double *x, int n2, double *h, double *y);

//         
double Hamming(int M, int i);

//         
double Blackman(int M, int i);

//     -
void LowPassFIRKernel(int M, double fc, bool IsHamming, double *kernel);

//     -
void HighPassFIRKernel(int M, double fc, bool IsHamming, double *kernel);

//      -
void BandPassFIRKernel(int M, double fc1, double fc2, bool IsHamming, double *kernel);

//      -
void BandRejectFIRKernel(int M, double fc1, double fc2, bool IsHamming, double *kernel);

//      -
void UserFIRKernel(int M, int *bands, int *freqs, int SmpRate, bool IsHamming, double *kernel);

//          
void GetPairCoeff(double fc, bool IsLopass, double ripple, int p, int poles, double& a0, double& a1, double& a2, double& b1, double& b2);

//      
void GetChebyshevCoeff(double fc, int poles, double ripple, bool IsLopass, double *a, double *b);

//   -
void IIRFilter(SampleData s, int n1, double *x, int n2, double *y);

//  -     
void FIRFilter(SampleData s, int n, double *kernel);

//  -      
void FFTFIRFilter(SampleData s, int n, double *kernel);

//  -         
void HomoFFTFIRFilter(SampleData s, int n, double *kernel);

//     " "
void MovingAverageFilter(SampleData s, int N);

//    
void RemoveDCOffset(SampleData s);

//   ""
void Echo(SampleData s, double delay, double decay, double input_amp, double echo_amp, bool IsMultiple, int smp_sec);

//   ""
void Reverberation(SampleData s, double InputAmp, double EarlyAmp, double ReverbAmp, double *early, double *ea, double *tail, double PreDelay, double ReverbTime, int smp_sec);

//   " "
void AmplitudeModulation(SampleData s, double freq, double phase, double depth, int smp_sec, int waveform);

//   ""
void WaveSynth(SampleData s, double freq, double phase, double depth, int smp_sec, int waveform);

//   
void AddNoise(SampleData s, double freq, double val, int smprate);

//   ""
void Fade(SampleData s, int FadeType, bool IsOut);

//   "  "
void PitchShift(SampleData s, int ShiftBy);

//   ""
void Chorus(SampleData s, int ChorusSize, int ChorusDelay, double ModFreq, double ModDepth, double InputAmp, double ChorusAmp, int smprate);

//   ""
void Reverse(SampleData s);

//       double
//   
void SamplesToDouble(SampleData s, double *x, double scale);

//      
//    (  )
void GetSpectrum(SampleData s, double *spectrum);

//       
double LevelTo_dB(double level, double cmp_level, double min_dB);

//     
int GetEqualizerFreq(int n, int HalfRate);

//        
int GetSampleRate(int index);

//         
void GetPoleZeroCoeff(double *rp, double *wp, double *r0, double *w0, double *a, double *b, int poles, int zeros, bool Normalize);

//       
void GetImpulseResponse(int num_a, double *a, int num_b, double *b, int length, double *data);

//       Z-   
Complex GetTransferFuncValue(Complex z, double *rp, double *wp, double *r0, double *w0, int poles, int zeros);

void SetProgressDlg(CProgressDlg *pd);

int GetTransformFlag();

void SetTransformFlag(int tf);
//-----------------------------------------------------------

#endif
