#ifndef _SCENE_H_
#define _SCENE_H_

#include <math.h>
#include "stdafx.h"
#include "RectDlg.h"
#include "SoundSrcDlg.h"
#include "vector.h"
#include "ray.h"

#define TYPE_LISTENER	1
#define TYPE_RECTANGLE	2
#define TYPE_SOUND		3

#define LISTENER_R		10
#define SOUND_R			17

#define SEL_COLOR		RGB(0xFF, 0, 0)
#define NORM_COLOR		RGB(0, 0, 0)

#define THRESHOLD		0.001
#define SOUND_SPEED		300.0

#define M_PI			3.14159265358979323846

struct SceneParams {
	double max_x;
	double max_y;
	double grid_x;
	double grid_y;
	bool use_grid;
};

struct SceneProperty {
	double AbLimit;
	double AmpLimit;
	double GroupSize;
	bool IgnoreAb;
	bool IgnoreSpec;
	bool Normalize;
	int NumRays;
	double SceneAb;
	double TimeLimit;
};

struct RayParams {
	double time;
	double amp;
	RayParams *next;
	RayParams()
	{
		time = amp = 0.0;
		next = NULL;
	}
	RayParams(double t, double a)
	{
		time = t;
		amp = a;
		next = NULL;
	}
	~RayParams()
	{
		if (next) delete next;
	}
};

class CSceneObject {
protected:
	double x, y, x_, y_;
	bool moving, selected;
	SceneParams *p;
	COLORREF color;
public:
	CSceneObject()
	{ 
		moving = false;
		Deselect();
	}
	CSceneObject(SceneParams *params)
	{
		moving = false;
		Deselect();
		p = params;
	}
	~CSceneObject() { }
	CSceneObject(double x, double y, SceneParams *p)
	{
		CSceneObject::x = x;
		CSceneObject::y = y;
		CSceneObject::p = p;
		moving = false;
		Deselect();
	}
	virtual bool Intersect(Ray ray, double &dist) { return false; }
	Vector GetOrigin()
	{
		return Vector(x, y);
	}
	void Select()
	{
		selected = true;
		color = SEL_COLOR;
	}
	void Deselect()
	{
		selected = false;
		color = NORM_COLOR;
	}
	virtual void PropertyDlg(CWnd *parent) = 0;
	virtual void MoveTo(double x2, double y2)
	{
		x = x2;
		y = y2;
		AlignToGrid(x, y);
		CheckPosition(x, y);
	}
	virtual void MoveBy(double cx, double cy)
	{
		moving = true;
		x += cx;
		y += cy;
		CheckPosition(x, y);
	}
	virtual void EndMove()
	{
		AlignToGrid(x, y);
		moving = false;
	}
	void AlignToGrid(double &x, double &y)
	{
		if (p->use_grid)
		{
			double x1, y1, gx = p->grid_x, gy = p->grid_y;
			x1 = floor(x / gx) * gx;
			y1 = floor(y / gy) * gy;
			if (x - x1 > 0.5 * gx) x = x1 + gx;
			else x = x1;
			if (y - y1 > 0.5 * gy) y = y1 + gy;
			else y = y1;
		}
	}
	virtual void CheckPosition(double &x, double &y)
	{
		if (x < 0) x = 0;
		if (y < 0) y = 0;
		if (x > p->max_x) x = p->max_x;
		if (y > p->max_y) y = p->max_y;
	}
	virtual double GetDistance(double x2, double y2)
	{
		return sqrt((x - x2) * (x - x2) + (y - y2) * (y - y2));
	}
	virtual int GetType() = 0;
	virtual bool PtInObj(double x, double y, double scale) = 0;
	virtual void Draw(CDC *dc, double scale)
	{
		x_ = x;
		y_ = y;
		if (moving) AlignToGrid(x_, y_);
	}
	virtual void Save(CArchive &ar)
	{
		ar.Write(&x, sizeof(double));
		ar.Write(&y, sizeof(double));
	}
	virtual void Load(CArchive &ar)
	{
		ar.Read(&x, sizeof(double));
		ar.Read(&y, sizeof(double));
	}
};


class CListener: public CSceneObject {
public:
	CListener(SceneParams *p): CSceneObject(p) { }
	CListener(double x, double y, SceneParams *p)
		:CSceneObject(x, y, p) { }
	int GetType() { return TYPE_LISTENER; }
	bool PtInObj(double x, double y, double scale)
	{
		if (GetDistance(x, y) < LISTENER_R / scale) return true;
		return false;
	}
	void PropertyDlg(CWnd *parent)
	{
		parent->MessageBox("    ");
	}
	void Draw(CDC *dc, double scale)
	{
		CSceneObject::Draw(dc, scale);
		int x = int(this->x_ * scale);
		int y = int(this->y_ * scale);
		int r = LISTENER_R;

		CPen pen(PS_SOLID, 2, color);
		dc->SelectObject(pen);
		dc->SelectStockObject(WHITE_BRUSH);
		dc->Ellipse(x - r, y - r, x + r, y + r);
		dc->SelectStockObject(BLACK_PEN);
	}
};

class CSound: public CSceneObject {
	double power;
public:
	CSound(SceneParams *p): CSceneObject(p) { }
	CSound(double x, double y, SceneParams *p)
		:CSceneObject(x, y, p) 
	{
		power = 1.0;
	}
	int GetType() { return TYPE_SOUND; }
	bool PtInObj(double x, double y, double scale)
	{
		if (GetDistance(x, y) < SOUND_R / scale) return true;
		return false;
	}
	void PropertyDlg(CWnd *parent)
	{
		CSoundSrcDlg d;
		d.m_Power = power * 100;
		if (d.DoModal() != IDOK) return;
		power = d.m_Power * 0.01;
	}
	void Draw(CDC *dc, double scale)
	{
		CSceneObject::Draw(dc, scale);
		int x = int(this->x_ * scale);
		int y = int(this->y_ * scale);

		CPen pen(PS_SOLID, 2, color);
		dc->SelectObject(pen);
		dc->MoveTo(x - 7, y - 7);
		dc->LineTo(x - 7, y + 7);
		dc->LineTo(x + 7, y + 7);
		dc->LineTo(x + 17, y + 17);
		dc->LineTo(x + 17, y - 17);
		dc->LineTo(x + 7, y - 7);
		dc->LineTo(x - 7, y - 7);
		dc->MoveTo(x + 7, y - 7);
		dc->LineTo(x + 7, y + 7);
		dc->SelectStockObject(BLACK_PEN);
	}
	void Save(CArchive &ar)
	{
		CSceneObject::Save(ar);
		ar.Write(&power, sizeof(double));
	}
	void Load(CArchive &ar)
	{
		CSceneObject::Load(ar);
		ar.Read(&power, sizeof(double));
	}
	double GetPower() { return power; }
};

class CRectangle: public CSceneObject {
	double w, h;
	double ac, sc;
public:
	CRectangle(SceneParams *p): CSceneObject(p) { }
	CRectangle(double x, double y, SceneParams *p)
		:CSceneObject(x, y, p)
	{
		w = h = 0.0;
		ac = 1.7E+308;
		sc = 0.0;
	}
	CRectangle(double x, double y, double w, double h, SceneParams *p)
		:CSceneObject(x, y, p)
	{
		CRectangle::w = w;
		CRectangle::h = h;
		ac = 1.7E+308;
		sc = 0.0;
	}
	bool HIntersect(double y, Ray ray, double &dist)
	{
		if (!ray.dir.y) return false;
		if (ray.dir.y > 0 && ray.org.y > y) return false;
		if (ray.dir.y < 0 && ray.org.y < y) return false;
		Vector p;
		dist = fabs((ray.org.y - y) / ray.dir.y);
		p = ray.Point(dist);
		if (p.x < x || p.x > x + w) return false;
		return true;
	}
	bool VIntersect(double x, Ray ray, double &dist)
	{
		if (!ray.dir.x) return false;
		if (ray.dir.x > 0 && ray.org.x > x) return false;
		if (ray.dir.x < 0 && ray.org.x < x) return false;
		Vector p;
		dist = fabs((ray.org.x - x) / ray.dir.x);
		p = ray.Point(dist);
		if (p.y < y || p.y > y + h) return false;
		return true;
	}
	virtual bool Intersect(Ray ray, double &dist)
	{
		double d[4], min = 1.7e+308;
		bool b[4], res = false;
		b[0] = HIntersect(y, ray, d[0]);
		b[1] = HIntersect(y + h, ray, d[1]);
		b[2] = VIntersect(x, ray, d[2]);
		b[3] = VIntersect(x + w, ray, d[3]);
		for (int i = 0; i < 4; i++)
		{
			res |= b[i];
			if (b[i] && d[i] < min) min = d[i];
		}
		dist = min;
		return res;
	}
	Vector GetNormal(Vector point)
	{
		if (point.x == x) return Vector(-1.0, 0.0);
		if (point.x == x + w) return Vector(1.0, 0.0);
		if (point.y == y) return Vector(0.0, -1.0);
		if (point.y == y + h) return Vector(0.0, 1.0);
		return Vector(0.0);
	}
	int GetType() { return TYPE_RECTANGLE; }
	void PropertyDlg(CWnd *parent)
	{
		CRectDlg d;
		d.m_AbsorbCoef = ac;
		d.m_ScepCoef = sc * 100;
		if (d.DoModal() != IDOK) return;
		ac = d.m_AbsorbCoef;
		sc = d.m_ScepCoef * 0.01;
	}
	void EndMove()
	{
		CSceneObject::EndMove();
		AlignToGrid(w, h);
	}
	void SetPoints(double x1, double y1, double x2, double y2)
	{
		CheckPosition(x2, y2);
		AlignToGrid(x1, y1);
		AlignToGrid(x2, y2);
		if (x2 < x1)
		{
			w = x1 - x2;
			x = x2;
		}
		else
		{
			w = x2 - x1;
			x = x1;
		}
		if (y2 < y1)
		{
			h = y1 - y2;
			y = y2;
		}
		else
		{
			h = y2 - y1;
			y = y1;
		}
	}
	bool PtInObj(double x_, double y_, double)
	{
		if (x_ >= x && x_ < x + w &&
			y_ >= y && y_ < y + h) return true;
		return false;
	}
	void MoveBy(double cx, double cy)
	{
		CSceneObject::MoveBy(cx, cy);
		if (x < 0) x = 0;
		if (y < 0) y = 0;
		if (x + w > p->max_x) x = p->max_x - w;
		if (y + h > p->max_y) y = p->max_y - h;
	}
	void Draw(CDC *dc, double scale)
	{
		CSceneObject::Draw(dc, scale);
		CPen pen(PS_SOLID, 2, color);
		dc->SelectObject(pen);
		int x_1 = int(x_ * scale);
		int x_2 = int((x_ + w) * scale);
		int y_1 = int(y_ * scale);
		int y_2 = int((y_ + h) * scale);
		dc->MoveTo(x_1, y_1);
		dc->LineTo(x_2, y_1);
		dc->LineTo(x_2, y_2);
		dc->LineTo(x_1, y_2);
		dc->LineTo(x_1, y_1);
		dc->SelectStockObject(BLACK_PEN);
	}
	void Save(CArchive &ar)
	{
		CSceneObject::Save(ar);
		ar.Write(&w, sizeof(double));
		ar.Write(&h, sizeof(double));
		ar.Write(&ac, sizeof(double));
		ar.Write(&sc, sizeof(double));
	}
	void Load(CArchive &ar)
	{
		CSceneObject::Load(ar);
		ar.Read(&w, sizeof(double));
		ar.Read(&h, sizeof(double));
		ar.Read(&ac, sizeof(double));
		ar.Read(&sc, sizeof(double));
	}
	double GetWidth() { return w; }
	double GetHeight() { return h; }
	double GetAC() { return ac; }
	double GetSC() { return sc; }
};

struct ObjList {
	CSceneObject *obj;
	ObjList *next, *prev;
};

class CObjList {
	ObjList *beg, *end;
public:
	CObjList()
	{
		beg = NULL;
		end = NULL;
	}
	~CObjList()
	{
		ClearList();
	}
	void AddObject(CSceneObject *obj)
	{
		ObjList *l = new ObjList;

		l->obj = obj;
		l->next = NULL;

		if (!beg)
		{
			beg = l;
			l->prev = NULL;
		}
		else
		{
			end->next = l;
			l->prev = end;
		}
		end = l;
	}
	void ClearList()
	{
		ObjList *l = beg;

		while (l)
		{
			beg = beg->next;
			delete l->obj;
			delete l;
			l = beg;
		}

		beg = end = NULL;
	}
	void DrawAll(CDC *dc, double scale)
	{
		ObjList *l = beg;

		while (l)
		{
			l->obj->Draw(dc, scale);
			l = l->next;
		}
	}
	void DeleteObject(CSceneObject *obj)
	{
		ObjList *l = beg;

		while (l)
		{
			if (l->obj == obj)
			{
				if (l == beg)
				{
					beg = l->next;
					beg->prev = NULL;
				}
				else
				if (l == end)
				{
					end = l->prev;
					end->next = NULL;
				}
				else
				{
					l->prev->next = l->next;
					l->next->prev = l->prev;
				}
				delete l->obj;
				delete l;
				return;
			}
			l = l->next;
		}
	}
	CSceneObject *SelectObject(CSceneObject *prev, double x, double y, double scale)
	{
		ObjList *l = beg;
		
		if (prev)
		while (l)
		{
			if (l->obj == prev)
			{
				l = l->next;
				break;
			}
			l = l->next;
		}

		if (!l) l = beg;

		while (l)
		{
			if (l->obj->PtInObj(x, y, scale))
				return l->obj;
			else if (l->obj == prev) return NULL;
			l = l->next;
			if (!l && prev) l = beg;
		}

		return NULL;
	}
	CSceneObject *SelectNext(CSceneObject *prev)
	{
		ObjList *l = beg;

		if (!prev) return beg->obj;
		if (prev == end->obj) return beg->obj;

		while (l)
		{
			if (l->obj == prev) return l->next->obj;
			l = l->next;
		}

		return NULL;
	}
	CSceneObject *GetObject(int count)
	{
		ObjList *l = beg;
		int i = 0;

		while (l)
		{
			if (count == i) return l->obj;
			l = l->next;
			i++;
		}

		return NULL;
	}
	int GetNumObjects()
	{
		ObjList *l = beg;
		int i = 0;

		while (l)
		{
			l = l->next;
			i++;
		}

		return i;
	}
	CSceneObject *Intersect(Ray ray, CSceneObject *ignore, double &dist)
	{
		ObjList *l = beg;
		double d;
		dist = 1.7e+308;
		CSceneObject *obj = NULL;

		while (l)
		{
			if (l->obj != ignore)
				if (l->obj->Intersect(ray, d))
					if (d < dist) 
					{
						dist = d;
						obj = l->obj;
					}
			l = l->next;
		}

		return obj;
	}
};

struct SoundList {
	double as;
	double power;
	SoundList *next;
	~SoundList()
	{
		if (next) delete next;
	}
};

struct TreeNode {
	double ab;
	double ac;
	double c1;
	double c2;
	double dist;
	double amp;
	SoundList *s;
	TreeNode *b;
	TreeNode *c;
	TreeNode *prev;
	~TreeNode()
	{
		if (b) delete b;
		if (c) delete c;
		if (s) delete s;
	}
};

class CTracer {
//	CDC *dc;
	CProgressCtrl *progress;
	CObjList *obj_list;
	RayParams *r_beg, *r_end;
	double *group;
	SceneProperty *sp;
	double max_dist, min_amp;
	int N;
	void Trace(Ray ray, CRectangle *cur_obj, TreeNode *cur_node)
	{
		CRectangle *obj;
		TreeNode *node, *prev = cur_node->prev;
		double d, c, dist, amp;
		Ray ray1 = ray;
		Vector n, point;
		if (prev)
		{
			cur_node->dist = prev->dist;
			cur_node->amp = prev->amp;
			if (prev->b == cur_node)
			{
				dist = prev->ab;
				amp = prev->c1;
			}
			else
			{
				dist = prev->ac;
				amp = prev->c2;
			}
			cur_node->dist += dist;
			cur_node->amp *= amp;
		}
		else
		{
			cur_node->dist = 0;
			cur_node->amp = 1.0;
		}
		point = ray.Point(dist);
		ray1.org = point;
//		dc->LineTo(point.x * 30, point.y * 30);
		if (!sp->IgnoreSpec)
			cur_node->s = CreateSoundList(cur_node, point);
		else
		{
			cur_node->s = NULL;
		}
		if (cur_node->dist > max_dist || cur_node->amp < min_amp)
		{
			cur_node->b = NULL;
			cur_node->c = NULL;
			return;
		}
		n = cur_obj->GetNormal(point);
		if (cur_obj->GetAC() < sp->AbLimit && !sp->IgnoreAb)
		{
			if ((obj = trace2(ray1, cur_obj, d, c)) != NULL)
			{
				node = new TreeNode;
				node->prev = cur_node;
				cur_node->c = node;
				cur_node->c2 = c;
				cur_node->ac = d;
//				dc->MoveTo(ray1.org.x * 30, ray1.org.y * 30);
				Trace(ray1, obj, node);
			}
			else
				cur_node->c = NULL;
		}
		else
			cur_node->c = NULL;
		ray1.dir = ray.dir - n * (2 * (ray.dir & n));
		if (cur_obj->GetSC() && !sp->IgnoreSpec)
		{
			if ((obj = trace1(ray1, cur_obj, d, c)) != NULL)
			{
				if ((obj->GetNormal(ray1.Point(d)) & -ray1.dir) < 1 - THRESHOLD)
				{
					node = new TreeNode;
					node->prev = cur_node;
					cur_node->b = node;
					cur_node->c1 = c * cur_obj->GetSC();
					cur_node->ab = d;
//					dc->MoveTo(ray1.org.x * 30, ray1.org.y * 30);
					Trace(ray1, obj, node);
				}
				else
					cur_node->b = NULL;
			}
			else
				cur_node->b = NULL;
		}
		else
			cur_node->b = NULL;
	}
	CRectangle *trace1(Ray ray, CRectangle *ignore, double &d, double &c)
	{
		CSceneObject *obj = obj_list->Intersect(ray, ignore, d);
		c = exp(d * sp->SceneAb);
		return (CRectangle *)obj;
	}
	CRectangle *trace2(Ray ray, CRectangle *cur_obj, double &d, double &c)
	{
		double d1, d2, d3;
		ray.org += ray.dir * THRESHOLD;
		cur_obj->Intersect(ray, d1);
		ray.org += ray.dir * d1;
		CRectangle *obj = trace1(ray, cur_obj, d2, c);
		d3 = d2;
		while (obj && d3 < THRESHOLD)
		{
			ray.org += ray.dir * (THRESHOLD + d3);
			obj = trace1(ray, cur_obj, d3, c);
			d2 += d3 + THRESHOLD;
		}
		d = d1 + d3 + THRESHOLD;
		c *= exp(-d1 * cur_obj->GetAC());
		return (CRectangle *)obj;
	}
	SoundList *CreateSoundList(TreeNode *node, Vector point)
	{
		SoundList *cur, *beg = NULL, *end = NULL;
		CSceneObject *obj;
		CRectangle *o;
		int i = 0;
		node->s = NULL;
		double d, c;
		Vector v;
		Ray r;

		while ((obj = obj_list->GetObject(i++)) != NULL)
		{
			if (obj->GetType() == TYPE_SOUND)
			{
				CSound *s = ((CSound *)obj);
				v = point - s->GetOrigin();
				r.org = s->GetOrigin();
				r.dir = v / !v;
				o = trace1(r, NULL, d, c);
				if (!o || fabs(d - !v) < THRESHOLD)
				{
//					dc->MoveTo(point.x * 30, point.y * 30);
//					dc->LineTo((s->GetOrigin()).x * 30, (s->GetOrigin()).y * 30);
					cur = new SoundList;
					cur->next = NULL;
					cur->power = s->GetPower();
					if (o) 
					{
						cur->power *= -r.dir & o->GetNormal(point);
						cur->power *= o->GetSC();
					}
					cur->as = !(point - s->GetOrigin());
					cur->power *= exp(-cur->as * sp->SceneAb);
					if (!beg)
						beg = cur;
					else
						end->next = cur;
					end = cur;
				}
			}
		}
		return beg;
	}
	void BeginTrace(Vector org)
	{
		Ray r;
		r.org = org;
		double angle = 2 * M_PI / sp->NumRays, d, c;
		TreeNode *root, *root2;
		CRectangle *obj;

		root = new TreeNode;
		root->dist = 0.0;
		root->amp = 1.0;
		root->s = CreateSoundList(root, org);
		root->c = NULL;
		root->b = NULL;
		MakeRaysFromNode(root);
		GroupLeafs();
		delete r_beg;
		r_beg = r_end = NULL;

		for (int i = 0; i < sp->NumRays; i++)
		{
			r.dir[0] = cos(i * angle);
			r.dir[1] = sin(i * angle);
			if ((obj = trace1(r, NULL, d, c)) != NULL)
			{
//				dc->MoveTo(org.x * 30, org.y * 30);
				root2 = new TreeNode;
				root->b = root2;
				root->ab = d;
				root->amp = 1.0;
				root->c1 = c;
				root2->prev = root;
				Trace(r, obj, root2);
//				MessageBox(0, "", "", 0);
				MakeRaysFromNode(root2);
				GroupLeafs();
				delete r_beg;
				r_beg = r_end = NULL;
				delete root2;
			}
			progress->SetPos(i * 100 / sp->NumRays);
		}

		root->b = NULL;
		delete root;
	}
	void AddRay(double dist, double amp)
	{
		RayParams *r = new RayParams(dist / SOUND_SPEED, amp);

		if (!r_beg)
			r_beg = r;
		else
			r_end->next = r;
		r_end = r;
	}
	void MakeRaysFromNode(TreeNode *node)
	{
		if (node == NULL) return;
		SoundList *s = node->s;
		double dist, amp;
		while (s)
		{
			dist = s->as + node->dist;
			amp = s->power * node->amp;
			if (dist < max_dist && amp > min_amp)
				AddRay(dist, amp);
			s = s->next;
		}
		MakeRaysFromNode(node->b);
		MakeRaysFromNode(node->c);
	}
	void GroupLeafs()
	{
		RayParams *r = r_beg;
		int i;

		while (r)
		{
			i = (int)floor(r->time / sp->GroupSize * 1000);
			group[i] += r->amp;
			r = r->next;
		}
	}
public:
	CTracer(SceneProperty *s, CObjList *ol, CProgressCtrl *p/*, CDC *DC*/)
	{
//		dc = DC;
		progress = p;
		sp = s;
		obj_list = ol;
		r_beg = r_end = NULL;
		max_dist = sp->TimeLimit * SOUND_SPEED * 0.001;
		min_amp = sp->AmpLimit;
	}
	void RenderScene(Vector org)
	{
		int i;
		N = (int)ceil(sp->TimeLimit / sp->GroupSize);
		group = new double[N];
		for (i = 0; i < N; i++)
			group[i] = 0.0;
		BeginTrace(org);
		if (sp->Normalize == 1)
		{
			double max = 0.0;
			for (i = 0; i < N; i++)
				if (group[i] > max) max = group[i];
			if (max > 1)
			for (i = 0; i < N; i++)
				group[i] /= max;
		}
	}
	double *GetRayGroup()
	{
		return group;
	}
	int GetGroupSize() { return N; }
};

class CScene {
	SceneParams p;
	SceneProperty sp;
	CObjList *obj_list;
	double *group;
	int g_size;
	void SetDefaults()
	{
		group = NULL;
		obj_list = new CObjList();
		sp.AbLimit = 1.6e+308;
		sp.AmpLimit = 0.001;
		sp.GroupSize = 10.0;
		sp.IgnoreAb = false;
		sp.IgnoreSpec = false;
		sp.Normalize = true;
		sp.NumRays = 100;
		sp.SceneAb = 0.0;
		sp.TimeLimit = 1000.0;
	}
public:
	CScene()
	{
		SetDefaults();
	}
	CScene(double size_x, double size_y)
	{
		p.max_x = size_x;
		p.max_y = size_y;
		p.use_grid = false;
		SetDefaults();
	}
	~CScene()
	{
		delete obj_list;
	}
	void AddListener(double x, double y)
	{
		obj_list->AddObject(new CListener(x, y, &p));
	}
	void AddObject(CSceneObject *obj)
	{
		obj_list->AddObject(obj);
	}
	CSceneObject *SelectObject(CSceneObject *prev, double x, double y, double scale)
	{
		if (prev) prev->Deselect();
		CSceneObject *s = obj_list->SelectObject(prev, x, y, scale);
		if (s) s->Select();
		return s;
	}
	CSceneObject *SelectNext(CSceneObject *prev)
	{
		if (prev) prev->Deselect();
		CSceneObject *s = obj_list->SelectNext(prev);
		if (s) s->Select();
		return s;
	}
	void DeleteObject(CSceneObject *obj)
	{
		if (obj) obj_list->DeleteObject(obj);
	}
	void DrawScene(CDC *dc, double scale)
	{
		obj_list->DrawAll(dc, scale);
	}
	bool SetSizes(double new_x, double new_y)
	{
		if (new_x == p.max_x && new_y == p.max_y)
			return false;

		p.max_x = new_x;
		p.max_y = new_y;

		return true;
	}
	bool SetGrid(double dx, double dy, bool use)
	{
		if (p.grid_x == dx && p.grid_y == dy && p.use_grid == use)
			return false;
		
		p.grid_x = dx;
		p.grid_y = dy;
		p.use_grid = use;

		return true;
	}
	int GetWidth() { return (int)p.max_x; }
	int GetHeight() { return (int)p.max_y; }
	int GetGridX() { return (int)p.grid_x; }
	int GetGridY() { return (int)p.grid_y; }
	bool GetAlign() { return p.use_grid; }
	SceneProperty *GetSceneProperty()
	{
		return &sp;
	}
	SceneParams *GetParams() { return &p; }
	void SaveAll(CArchive &ar)
	{
		int i = 0, t;
		CSceneObject *obj;
		CString sig = _T("SCENE");
		t = obj_list->GetNumObjects();
		ar << sig;
		ar.Write(&p, sizeof(SceneParams));
		ar.Write(&sp, sizeof(SceneProperty));
		ar.Write(&t, sizeof(int));
		while ((obj = obj_list->GetObject(i++)) != NULL)
		{
			t = obj->GetType();
			ar.Write(&t, sizeof(int));
			obj->Save(ar);
		}
	}
	void LoadAll(CArchive &ar)
	{
		obj_list->ClearList();
		int i = 0, num_obj, t;
		CSceneObject *obj;
		CString sig;
		ar >> sig;
		if (sig != "SCENE")
			AfxThrowArchiveException(CArchiveException::badIndex, ar.GetFile()->GetFileName());
		ar.Read(&p, sizeof(SceneParams));
		ar.Read(&sp, sizeof(SceneProperty));
		ar.Read(&num_obj, sizeof(int));
		for (; i < num_obj; i++)
		{
			ar.Read(&t, sizeof(int));
			switch (t)
			{
				case TYPE_LISTENER:
					obj = (CSceneObject *)(new CListener(&p));
					break;
				case TYPE_SOUND:
					obj = (CSceneObject *)(new CSound(&p));
					break;
				case TYPE_RECTANGLE:
					obj = (CSceneObject *)(new CRectangle(&p));
			}
			obj->Load(ar);
			AddObject(obj);
		}
	}
	bool CheckScene(char *message)
	{
		int n = obj_list->GetNumObjects(), i, j;
		CSceneObject *obj1, *obj2;

		for (i = 0; i < n; i++)
		{
			obj1 = obj_list->GetObject(i);
			for (j = 0; j < n; j++)
			{
				obj2 = obj_list->GetObject(j);
				if (obj2->GetType() == TYPE_RECTANGLE && obj1 != obj2)
				{
					Vector p = obj1->GetOrigin();
					if (obj2->PtInObj(p.x, p.y, 1.0))
					{
						if (obj1->GetType() != TYPE_RECTANGLE)
							sprintf(message, " (%g,%g)  ", p.x, p.y);
						else
							sprintf(message, " ");
						return false;
					}
					if (obj1->GetType() == TYPE_RECTANGLE)
					{
						double w = ((CRectangle *)obj1)->GetWidth();
						double h = ((CRectangle *)obj1)->GetHeight();
						w -= THRESHOLD;
						h -= THRESHOLD;
						if (obj2->PtInObj(p.x + w, p.y, 1.0) ||
							obj2->PtInObj(p.x, p.y + h, 1.0))
						{
							sprintf(message, " ");
							return false;
						}
					}
				}
			}
		}
		return true;
	}
	void CalcEffect(CProgressCtrl *progress/*CDC *dc*/)
	{
		CTracer *tracer = new CTracer(&sp, obj_list, progress/*, dc*/);
		if (group) delete group;
		Vector org = obj_list->GetObject(0)->GetOrigin();
		tracer->RenderScene(org);
		group = tracer->GetRayGroup();
		g_size = tracer->GetGroupSize();
		delete tracer;
	}
	void SaveEffect(CArchive &ar)
	{
		CString sig = _T("EFFECT");
		ar << sig;
		ar.Write(&g_size, sizeof(int));
		ar.Write(&sp.TimeLimit, sizeof(double));
		ar.Write(group, sizeof(double) * g_size);
	}
};

#endif
